﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("Multi-objective Evaluator", "Calls the script's Evaluate method to get the quality values of the parameter vector.")]
  [StorableClass]
  public class MultiObjectiveEvaluator : SingleSuccessorOperator, IMultiObjectiveProgrammableProblemEvaluator, IStochasticOperator {

    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }

    public ILookupParameter<IMultiObjectiveProblemDefinitionHost> ProblemDefinitionParameter {
      get { return (ILookupParameter<IMultiObjectiveProblemDefinitionHost>)Parameters["ProblemDefinition"]; }
    }

    public ILookupParameter<Configuration> ConfigurationParameter {
      get { return (ILookupParameter<Configuration>)Parameters["Configuration"]; }
    }

    public ILookupParameter<DoubleArray> QualitiesParameter {
      get { return (ILookupParameter<DoubleArray>)Parameters["Qualities"]; }
    }

    [StorableConstructor]
    protected MultiObjectiveEvaluator(bool deserializing) : base(deserializing) { }
    protected MultiObjectiveEvaluator(MultiObjectiveEvaluator original, Cloner cloner) : base(original, cloner) { }
    public MultiObjectiveEvaluator() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The random number generator to use."));
      Parameters.Add(new LookupParameter<IMultiObjectiveProblemDefinitionHost>("ProblemDefinition", "The host that holds the problem definition."));
      Parameters.Add(new LookupParameter<Configuration>("Configuration", "An item that holds the problem's configuration."));
      Parameters.Add(new LookupParameter<DoubleArray>("Qualities", "The qualities of the parameter vector."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MultiObjectiveEvaluator(this, cloner);
    }

    public override IOperation Apply() {
      var random = RandomParameter.ActualValue;
      var host = ProblemDefinitionParameter.ActualValue;
      if (host.Instance == null) throw new InvalidOperationException("Script instance is null, maybe the code doesn't compile.");
      var config = ConfigurationParameter.ActualValue;
      var vector = Helper.Extract(ExecutionContext.Scope, config);
      QualitiesParameter.ActualValue = new DoubleArray(host.Instance.Evaluate(random, vector));
      return base.Apply();
    }
  }
}
